export function reconstructTree(data: any[]) {
    // Step 1: Sort data by chained array to ensure parent-child order
    const sortedData = [...data].sort((a, b) => {
        const aChained = a.chained;
        const bChained = b.chained;
        for (let i = 0; i < Math.max(aChained.length, bChained.length); i++) {
            const aVal = aChained[i] || 0;
            const bVal = bChained[i] || 0;
            if (aVal !== bVal) return aVal - bVal;
        }
        return 0;
    });

    // Step 2: Build a map for quick lookup and to track children
    const nodeMap = new Map();
    const result: any[] = [];

    sortedData.forEach(item => {
        nodeMap.set(item.key, { ...item, children: [] });
    });

    // Step 3: Assign children to parents based on chained hierarchy
    sortedData.forEach(item => {
        const node = nodeMap.get(item.key);
        const chained = item.chained;

        // Find the parent by looking for the closest shorter chained array
        let parent = null;
        for (let i = chained.length - 1; i > 0; i--) {
            const parentChained = chained.slice(0, i);
            const parentKey = sortedData.find(d =>
                d.type !== 'placeholder' &&
                d.chained.length === parentChained.length &&
                d.chained.every((val: any, idx: number) => val === parentChained[idx])
            )?.key;

            if (parentKey) {
                parent = nodeMap.get(parentKey);
                break;
            }
        }

        if (parent) {
            parent.children.push(node);
        } else {
            result.push(node);
        }
    });

    // Step 4: Convert to XML
    function createXmlNode(node: any, doc: Document) {
        const element = doc.createElement(node.label);

        // Set data-key attribute
        element.setAttribute('data-key', node.key);

        // Set data-type attribute if type is not null
        if (node.type !== null) {
            element.setAttribute('data-modify-type', node.type);
        }

        element.setAttribute('data-w-e-type', node.label)
        element.setAttribute('data-indent-level', node.chained.length.toString())

        // Set textContent for TITLE or PARA nodes
        if (node.label === 'TITLE' || node.label === 'PARA') {
            element.textContent = node.textContent;
        }

        // Recursively add children
        node.children.forEach((child: any) => {
            element.appendChild(createXmlNode(child, doc));
        });

        return element;
    }

    // Create XML document
    const doc = new Document();

    // Use the first element (JOBCARD) as the root if it exists
    if (result.length > 0) {
        const rootNode = createXmlNode(result[0], doc)
        doc.appendChild(rootNode)
    }

    // Serialize to XML string
    const serializer = new XMLSerializer();
    return serializer.serializeToString(doc);
}